@@ -12,6 +12,7 @@ import colorama
 
 import borgmatic.actions.borg
 import borgmatic.actions.break_lock
+import borgmatic.actions.change_passphrase
 import borgmatic.actions.check
 import borgmatic.actions.compact
 import borgmatic.actions.config.bootstrap
@@ -26,11 +27,11 @@ import borgmatic.actions.info
 import borgmatic.actions.list
 import borgmatic.actions.mount
 import borgmatic.actions.prune
-import borgmatic.actions.rcreate
-import borgmatic.actions.rdelete
+import borgmatic.actions.repo_create
+import borgmatic.actions.repo_delete
+import borgmatic.actions.repo_info
+import borgmatic.actions.repo_list
 import borgmatic.actions.restore
-import borgmatic.actions.rinfo
-import borgmatic.actions.rlist
 import borgmatic.actions.transfer
 import borgmatic.commands.completion.bash
 import borgmatic.commands.completion.fish
@@ -38,7 +39,8 @@ from borgmatic.borg import umount as borg_umount
 from borgmatic.borg import version as borg_version
 from borgmatic.commands.arguments import parse_arguments
 from borgmatic.config import checks, collect, validate
-from borgmatic.hooks import command, dispatch, monitor
+from borgmatic.hooks import command, dispatch
+from borgmatic.hooks.monitoring import monitor
 from borgmatic.logger import DISABLED, add_custom_log_levels, configure_logging, should_do_markup
 from borgmatic.signals import configure_signals
 from borgmatic.verbosity import verbosity_to_log_level
@@ -102,7 +104,7 @@ def run_configuration(config_filename, config, config_paths, arguments):
                 'initialize_monitor',
                 config,
                 config_filename,
-                monitor.MONITOR_HOOK_NAMES,
+                dispatch.Hook_type.MONITORING,
                 monitoring_log_level,
                 global_arguments.dry_run,
             )
@@ -111,7 +113,7 @@ def run_configuration(config_filename, config, config_paths, arguments):
                 'ping_monitor',
                 config,
                 config_filename,
-                monitor.MONITOR_HOOK_NAMES,
+                dispatch.Hook_type.MONITORING,
                 monitor.State.START,
                 monitoring_log_level,
                 global_arguments.dry_run,
@@ -171,7 +173,7 @@ def run_configuration(config_filename, config, config_paths, arguments):
                     continue
 
                 if command.considered_soft_failure(config_filename, error):
-                    break
+                    continue
 
                 yield from log_error_records(
                     f'{repository.get("label", repository["path"])}: Error running actions for repository',
@@ -187,7 +189,7 @@ def run_configuration(config_filename, config, config_paths, arguments):
                 'ping_monitor',
                 config,
                 config_filename,
-                monitor.MONITOR_HOOK_NAMES,
+                dispatch.Hook_type.MONITORING,
                 monitor.State.LOG,
                 monitoring_log_level,
                 global_arguments.dry_run,
@@ -204,7 +206,7 @@ def run_configuration(config_filename, config, config_paths, arguments):
                     'ping_monitor',
                     config,
                     config_filename,
-                    monitor.MONITOR_HOOK_NAMES,
+                    dispatch.Hook_type.MONITORING,
                     monitor.State.FINISH,
                     monitoring_log_level,
                     global_arguments.dry_run,
@@ -213,7 +215,7 @@ def run_configuration(config_filename, config, config_paths, arguments):
                     'destroy_monitor',
                     config,
                     config_filename,
-                    monitor.MONITOR_HOOK_NAMES,
+                    dispatch.Hook_type.MONITORING,
                     monitoring_log_level,
                     global_arguments.dry_run,
                 )
@@ -240,7 +242,7 @@ def run_configuration(config_filename, config, config_paths, arguments):
                 'ping_monitor',
                 config,
                 config_filename,
-                monitor.MONITOR_HOOK_NAMES,
+                dispatch.Hook_type.MONITORING,
                 monitor.State.FAIL,
                 monitoring_log_level,
                 global_arguments.dry_run,
@@ -249,7 +251,7 @@ def run_configuration(config_filename, config, config_paths, arguments):
                 'destroy_monitor',
                 config,
                 config_filename,
-                monitor.MONITOR_HOOK_NAMES,
+                dispatch.Hook_type.MONITORING,
                 monitoring_log_level,
                 global_arguments.dry_run,
             )
@@ -306,8 +308,8 @@ def run_actions(
     )
 
     for action_name, action_arguments in arguments.items():
-        if action_name == 'rcreate' and action_name not in skip_actions:
-            borgmatic.actions.rcreate.run_rcreate(
+        if action_name == 'repo-create' and action_name not in skip_actions:
+            borgmatic.actions.repo_create.run_repo_create(
                 repository,
                 config,
                 local_borg_version,
@@ -421,8 +423,8 @@ def run_actions(
                 local_path,
                 remote_path,
             )
-        elif action_name == 'rlist' and action_name not in skip_actions:
-            yield from borgmatic.actions.rlist.run_rlist(
+        elif action_name == 'repo-list' and action_name not in skip_actions:
+            yield from borgmatic.actions.repo_list.run_repo_list(
                 repository,
                 config,
                 local_borg_version,
@@ -441,8 +443,8 @@ def run_actions(
                 local_path,
                 remote_path,
             )
-        elif action_name == 'rinfo' and action_name not in skip_actions:
-            yield from borgmatic.actions.rinfo.run_rinfo(
+        elif action_name == 'repo-info' and action_name not in skip_actions:
+            yield from borgmatic.actions.repo_info.run_repo_info(
                 repository,
                 config,
                 local_borg_version,
@@ -481,6 +483,16 @@ def run_actions(
                 local_path,
                 remote_path,
             )
+        elif action_name == 'change-passphrase' and action_name not in skip_actions:
+            borgmatic.actions.change_passphrase.run_change_passphrase(
+                repository,
+                config,
+                local_borg_version,
+                action_arguments,
+                global_arguments,
+                local_path,
+                remote_path,
+            )
         elif action_name == 'delete' and action_name not in skip_actions:
             borgmatic.actions.delete.run_delete(
                 repository,
@@ -491,8 +503,8 @@ def run_actions(
                 local_path,
                 remote_path,
             )
-        elif action_name == 'rdelete' and action_name not in skip_actions:
-            borgmatic.actions.rdelete.run_rdelete(
+        elif action_name == 'repo-delete' and action_name not in skip_actions:
+            borgmatic.actions.repo_delete.run_repo_delete(
                 repository,
                 config,
                 local_borg_version,
@@ -579,7 +591,7 @@ def load_configurations(config_filenames, overrides=None, resolve_env=True):
                         )
                     ),
                     logging.makeLogRecord(
-                        dict(levelno=logging.CRITICAL, levelname='CRITICAL', msg=error)
+                        dict(levelno=logging.CRITICAL, levelname='CRITICAL', msg=str(error))
                     ),
                 ]
             )
@@ -600,6 +612,9 @@ def log_record(suppress_log=False, **kwargs):
     return record
 
 
+BORG_REPOSITORY_ACCESS_ABORTED_EXIT_CODE = 62
+
+
 def log_error_records(
     message, error=None, levelno=logging.CRITICAL, log_command_error_output=False
 ):
@@ -614,13 +629,13 @@ def log_error_records(
     level_name = logging._levelToName[levelno]
 
     if not error:
-        yield log_record(levelno=levelno, levelname=level_name, msg=message)
+        yield log_record(levelno=levelno, levelname=level_name, msg=str(message))
         return
 
     try:
         raise error
     except CalledProcessError as error:
-        yield log_record(levelno=levelno, levelname=level_name, msg=message)
+        yield log_record(levelno=levelno, levelname=level_name, msg=str(message))
 
         if error.output:
             try:
@@ -639,10 +654,17 @@ def log_error_records(
                     suppress_log=True,
                 )
 
-        yield log_record(levelno=levelno, levelname=level_name, msg=error)
+        yield log_record(levelno=levelno, levelname=level_name, msg=str(error))
+
+        if error.returncode == BORG_REPOSITORY_ACCESS_ABORTED_EXIT_CODE:
+            yield log_record(
+                levelno=levelno,
+                levelname=level_name,
+                msg='\nTo work around this, set either the "relocated_repo_access_is_ok" or "unknown_unencrypted_repo_access_is_ok" option to "true", as appropriate.',
+            )
     except (ValueError, OSError) as error:
-        yield log_record(levelno=levelno, levelname=level_name, msg=message)
-        yield log_record(levelno=levelno, levelname=level_name, msg=error)
+        yield log_record(levelno=levelno, levelname=level_name, msg=str(message))
+        yield log_record(levelno=levelno, levelname=level_name, msg=str(error))
     except:  # noqa: E722
         # Raising above only as a means of determining the error type. Swallow the exception here
         # because we don't want the exception to propagate out of this function.
@@ -673,7 +695,9 @@ def collect_highlander_action_summary_logs(configs, arguments, configuration_par
     if 'bootstrap' in arguments:
         try:
             # No configuration file is needed for bootstrap.
-            local_borg_version = borg_version.local_borg_version({}, 'borg')
+            local_borg_version = borg_version.local_borg_version(
+                {}, arguments['bootstrap'].local_path
+            )
         except (OSError, CalledProcessError, ValueError) as error:
             yield from log_error_records('Error getting local Borg version', error)
             return
@@ -770,9 +794,6 @@ def collect_configuration_run_summary_logs(configs, config_paths, arguments):
             break
 
     try:
-        if 'extract' in arguments or 'mount' in arguments:
-            validate.guard_single_repository_selected(repository, configs)
-
         validate.guard_configuration_contains_repository(repository, configs)
     except ValueError as error:
         yield from log_error_records(str(error))
